from abc import ABC, abstractmethod
from typing import Iterable


class UserContextInterface(ABC):

    @property
    @abstractmethod
    def id(self) -> str: ...

    @property
    @abstractmethod
    def roles(self) -> Iterable[str]: ...



from datetime import datetime
from typing import Protocol, ClassVar

class BaseEntityInterface(Protocol):
    PK_FIELD: ClassVar[str]

    @property
    def id(self) -> str: ...

    @property
    def created_at(self) -> datetime: ...

    @property
    def updated_at(self) -> datetime: ...


from abc import ABC, abstractmethod
from datetime import datetime


class BaseModelInterface(ABC):

    @property
    @abstractmethod
    def id(self) -> str: ...

    @property
    @abstractmethod
    def created_at(self) -> datetime: ...

    @property
    @abstractmethod
    def updated_at(self) -> datetime: ...



from dataclasses import dataclass, field
from datetime import datetime
import uuid

from architectonics.common.interfaces.base_model_interface import BaseModelInterface


@dataclass
class BaseModel(BaseModelInterface):
    id: str = field(
        default_factory=lambda: str(
            uuid.uuid4(),
        ),
    )

    created_at: datetime = field(
        default_factory=datetime.utcnow,
    )

    updated_at: datetime = field(
        default_factory=datetime.utcnow,
    )
